==> rsuite-4.5.0.tgz: package/es/Overlay/OverlayTrigger.js <==
import _extends from "@babel/runtime/helpers/esm/extends";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import * as React from 'react';
import get from 'lodash/get';
import pick from 'lodash/pick';
import { contains } from 'dom-lib';
import Overlay from './Overlay';
import createChainedFunction from '../utils/createChainedFunction';
import isOneOf from '../utils/isOneOf';
import getDOMNode from '../utils/getDOMNode';
import Portal from '../Portal';

function isNullOrUndefined(value) {
  return value === null || typeof value === 'undefined';
}

function onMouseEventHandler(handler, event) {
  var target = event.currentTarget;
  var related = event.relatedTarget || get(event, ['nativeEvent', 'toElement']);

  if ((!related || related !== target) && !contains(target, related)) {
    handler(event);
  }
}

var OverlayTrigger =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(OverlayTrigger, _React$Component);

  function OverlayTrigger(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.onMouseOverListener = void 0;
    _this.onMouseOutListener = void 0;
    _this.hoverShowDelayTimer = void 0;
    _this.hoverHideDelayTimer = void 0;
    _this.mouseEnteredToSpeaker = false;
    _this.mouseEnteredToTrigger = false;

    _this.getOverlayTarget = function () {
      return getDOMNode(_assertThisInitialized(_this));
    };

    _this.handleSpeakerMouseEnter = function () {
      _this.mouseEnteredToSpeaker = true;
    };

    _this.handleSpeakerMouseLeave = function () {
      var trigger = _this.props.trigger;
      _this.mouseEnteredToSpeaker = false;

      if (!isOneOf('click', trigger) && !isOneOf('active', trigger)) {
        _this.handleHide();
      }
    };

    _this.hide = function () {
      _this.setState({
        isOverlayShown: false
      });
    };

    _this.show = function () {
      _this.setState({
        isOverlayShown: true
      });
    };

    _this.handleHide = function () {
      if (!_this.mouseEnteredToSpeaker && !_this.mouseEnteredToTrigger) {
        _this.hide();
      }
    };

    _this.handleToggle = function () {
      if (_this.state.isOverlayShown) {
        _this.handleHide();
      } else {
        _this.show();
      }
    };

    _this.handleDelayedShow = function () {
      var _this$props = _this.props,
          delayShow = _this$props.delayShow,
          delay = _this$props.delay;
      _this.mouseEnteredToTrigger = true;

      if (!isNullOrUndefined(_this.hoverHideDelayTimer)) {
        clearTimeout(_this.hoverHideDelayTimer);
        _this.hoverHideDelayTimer = null;

        _this.show();

        return;
      }

      if (_this.state.isOverlayShown) {
        return;
      }

      var nextDelay = !isNullOrUndefined(delayShow) ? delayShow : delay;

      if (!nextDelay) {
        _this.show();

        return;
      }

      _this.hoverShowDelayTimer = setTimeout(function () {
        _this.hoverShowDelayTimer = null;

        _this.show();
      }, nextDelay);
    };

    _this.handleDelayedHide = function () {
      var _this$props2 = _this.props,
          delayHide = _this$props2.delayHide,
          delay = _this$props2.delay;
      _this.mouseEnteredToTrigger = false;

      if (!isNullOrUndefined(_this.hoverShowDelayTimer)) {
        clearTimeout(_this.hoverShowDelayTimer);
        _this.hoverShowDelayTimer = null;
        return;
      }

      if (!_this.state.isOverlayShown || !isNullOrUndefined(_this.hoverHideDelayTimer)) {
        return;
      }

      var nextDelay = !isNullOrUndefined(delayHide) ? delayHide : delay;

      if (!nextDelay) {
        _this.handleHide();

        return;
      }

      _this.hoverHideDelayTimer = setTimeout(function () {
        if (_this.state.isOnSpeaker) {
          return;
        }

        clearTimeout(_this.hoverHideDelayTimer);
        _this.hoverHideDelayTimer = null;

        _this.handleHide();
      }, nextDelay);
    };

    _this.onMouseOverListener = function (e) {
      return onMouseEventHandler(_this.handleDelayedShow, e);
    };

    _this.onMouseOutListener = function (e) {
      return onMouseEventHandler(_this.handleDelayedHide, e);
    };

    _this.state = {
      isOverlayShown: props.defaultOpen
    };
    return _this;
  }

  var _proto = OverlayTrigger.prototype;

  _proto.componentWillUnmount = function componentWillUnmount() {
    clearTimeout(this.hoverShowDelayTimer);
    clearTimeout(this.hoverHideDelayTimer);
  };

  _proto.getOverlay = function getOverlay() {
    var _this$props3 = this.props,
        open = _this$props3.open,
        speaker = _this$props3.speaker,
        trigger = _this$props3.trigger,
        onHide = _this$props3.onHide;
    var isOverlayShown = this.state.isOverlayShown;

    var overlayProps = _extends({}, pick(this.props, Object.keys(Overlay.propTypes)), {
      show: typeof open === 'undefined' ? isOverlayShown : open,
      target: this.getOverlayTarget
    });

    if (isOneOf('click', trigger)) {
      overlayProps.onHide = createChainedFunction(this.hide, onHide);
    } else if (isOneOf('active', trigger)) {
      overlayProps.onHide = createChainedFunction(this.hide, onHide);
    }

    var speakerProps = {
      onMouseEnter: this.handleSpeakerMouseEnter,
      onMouseLeave: this.handleSpeakerMouseLeave,
      placement: overlayProps.placement
    };

    if (typeof speaker === 'function') {
      return React.createElement(Overlay, overlayProps, speaker);
    }

    return React.createElement(Overlay, overlayProps, React.cloneElement(speaker, speakerProps));
  };

  _proto.render = function render() {
    var _this$props4 = this.props,
        children = _this$props4.children,
        speaker = _this$props4.speaker,
        onClick = _this$props4.onClick,
        trigger = _this$props4.trigger,
        onMouseOver = _this$props4.onMouseOver,
        onMouseOut = _this$props4.onMouseOut,
        onFocus = _this$props4.onFocus,
        onBlur = _this$props4.onBlur,
        disabled = _this$props4.disabled;
    var triggerComponent = React.Children.only(children);
    var triggerProps = triggerComponent.props;
    var props = {
      key: 'triggerComponent',
      onClick: createChainedFunction(triggerProps.onClick, onClick),
      'aria-describedby': get(speaker, ['props', 'id'])
    };

    if (!disabled) {
      if (isOneOf('click', trigger)) {
        props.onClick = createChainedFunction(this.handleToggle, props.onClick);
      }

      if (isOneOf('active', trigger)) {
        props.onClick = createChainedFunction(this.show, props.onClick);
      }

      if (isOneOf('hover', trigger)) {
        props.onMouseOver = createChainedFunction(this.onMouseOverListener, triggerProps.onMouseOver, onMouseOver);
        props.onMouseOut = createChainedFunction(this.onMouseOutListener, triggerProps.onMouseOut, onMouseOut);
      }

      if (isOneOf('focus', trigger)) {
        props.onFocus = createChainedFunction(this.handleDelayedShow, triggerProps.onFocus, onFocus);
        props.onBlur = createChainedFunction(this.handleDelayedHide, triggerProps.onBlur, onBlur);
      }
    }

    return [React.cloneElement(triggerComponent, props), React.createElement(Portal, {
      key: "portal"
    }, this.getOverlay())];
  };

  return OverlayTrigger;
}(React.Component);

OverlayTrigger.defaultProps = {
  trigger: ['hover', 'focus'],
  delayHide: 200,
  placement: 'bottomStart',
  rootClose: true
};
export default OverlayTrigger;

==> rsuite-4.7.2.tgz: package/es/Overlay/OverlayTrigger.js <==
import _extends from "@babel/runtime/helpers/esm/extends";
import _assertThisInitialized from "@babel/runtime/helpers/esm/assertThisInitialized";
import _inheritsLoose from "@babel/runtime/helpers/esm/inheritsLoose";
import * as React from 'react';
import get from 'lodash/get';
import pick from 'lodash/pick';
import { contains } from 'dom-lib';
import Overlay from './Overlay';
import createChainedFunction from '../utils/createChainedFunction';
import isOneOf from '../utils/isOneOf';
import getDOMNode from '../utils/getDOMNode';
import Portal from '../Portal';

function isNullOrUndefined(value) {
  return value === null || typeof value === 'undefined';
}

function onMouseEventHandler(handler, event) {
  var target = event.currentTarget;
  var related = event.relatedTarget || get(event, ['nativeEvent', 'toElement']);

  if ((!related || related !== target) && !contains(target, related)) {
    handler(event);
  }
}

var OverlayTrigger =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(OverlayTrigger, _React$Component);

  function OverlayTrigger(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;
    _this.onMouseOverListener = void 0;
    _this.onMouseOutListener = void 0;
    _this.hoverShowDelayTimer = void 0;
    _this.hoverHideDelayTimer = void 0;
    _this.mouseEnteredToSpeaker = false;
    _this.mouseEnteredToTrigger = false;

    _this.getOverlayTarget = function () {
      return getDOMNode(_assertThisInitialized(_this));
    };

    _this.handleSpeakerMouseEnter = function () {
      _this.mouseEnteredToSpeaker = true;
    };

    _this.handleSpeakerMouseLeave = function () {
      var trigger = _this.props.trigger;
      _this.mouseEnteredToSpeaker = false;

      if (!isOneOf('click', trigger) && !isOneOf('active', trigger)) {
        _this.handleHide();
      }
    };

    _this.hide = function () {
      _this.setState({
        isOverlayShown: false
      });
    };

    _this.show = function () {
      _this.setState({
        isOverlayShown: true
      });
    };

    _this.handleHide = function () {
      if (!_this.mouseEnteredToSpeaker && !_this.mouseEnteredToTrigger) {
        _this.hide();
      }
    };

    _this.handleToggle = function () {
      if (_this.state.isOverlayShown) {
        _this.handleHide();
      } else {
        _this.show();
      }
    };

    _this.handleDelayedShow = function () {
      var _this$props = _this.props,
          delayShow = _this$props.delayShow,
          delay = _this$props.delay,
          enterable = _this$props.enterable;

      if (!enterable) {
        _this.show();

        return;
      }

      _this.mouseEnteredToTrigger = true;

      if (!isNullOrUndefined(_this.hoverHideDelayTimer)) {
        clearTimeout(_this.hoverHideDelayTimer);
        _this.hoverHideDelayTimer = null;

        _this.show();

        return;
      }

      if (_this.state.isOverlayShown) {
        return;
      }

      var nextDelay = !isNullOrUndefined(delayShow) ? delayShow : delay;

      if (!nextDelay) {
        _this.show();

        return;
      }

      _this.hoverShowDelayTimer = setTimeout(function () {
        _this.hoverShowDelayTimer = null;

        _this.show();
      }, nextDelay);
    };

    _this.handleDelayedHide = function () {
      var _this$props2 = _this.props,
          delayHide = _this$props2.delayHide,
          delay = _this$props2.delay,
          enterable = _this$props2.enterable;

      if (!enterable) {
        _this.hide();

        return;
      }

      _this.mouseEnteredToTrigger = false;

      if (!isNullOrUndefined(_this.hoverShowDelayTimer)) {
        clearTimeout(_this.hoverShowDelayTimer);
        _this.hoverShowDelayTimer = null;
        return;
      }

      if (!_this.state.isOverlayShown || !isNullOrUndefined(_this.hoverHideDelayTimer)) {
        return;
      }

      var nextDelay = !isNullOrUndefined(delayHide) ? delayHide : delay;

      if (!nextDelay) {
        _this.handleHide();

        return;
      }

      _this.hoverHideDelayTimer = setTimeout(function () {
        if (_this.state.isOnSpeaker) {
          return;
        }

        clearTimeout(_this.hoverHideDelayTimer);
        _this.hoverHideDelayTimer = null;

        _this.handleHide();
      }, nextDelay);
    };

    _this.onMouseOverListener = function (e) {
      return onMouseEventHandler(_this.handleDelayedShow, e);
    };

    _this.onMouseOutListener = function (e) {
      return onMouseEventHandler(_this.handleDelayedHide, e);
    };

    _this.state = {
      isOverlayShown: props.defaultOpen
    };
    return _this;
  }

  var _proto = OverlayTrigger.prototype;

  _proto.componentWillUnmount = function componentWillUnmount() {
    clearTimeout(this.hoverShowDelayTimer);
    clearTimeout(this.hoverHideDelayTimer);
  };

  _proto.getOverlay = function getOverlay() {
    var _this$props3 = this.props,
        open = _this$props3.open,
        speaker = _this$props3.speaker,
        trigger = _this$props3.trigger,
        onHide = _this$props3.onHide;
    var isOverlayShown = this.state.isOverlayShown;

    var overlayProps = _extends({}, pick(this.props, Object.keys(Overlay.propTypes)), {
      show: typeof open === 'undefined' ? isOverlayShown : open,
      target: this.getOverlayTarget
    });

    if (isOneOf('click', trigger)) {
      overlayProps.onHide = createChainedFunction(this.hide, onHide);
    } else if (isOneOf('active', trigger)) {
      overlayProps.onHide = createChainedFunction(this.hide, onHide);
    }

    var speakerProps = {
      onMouseEnter: this.handleSpeakerMouseEnter,
      onMouseLeave: this.handleSpeakerMouseLeave,
      placement: overlayProps.placement
    };

    if (typeof speaker === 'function') {
      return React.createElement(Overlay, overlayProps, speaker);
    }

    return React.createElement(Overlay, overlayProps, React.cloneElement(speaker, speakerProps));
  };

  _proto.render = function render() {
    var _this$props4 = this.props,
        children = _this$props4.children,
        speaker = _this$props4.speaker,
        onClick = _this$props4.onClick,
        trigger = _this$props4.trigger,
        onMouseOver = _this$props4.onMouseOver,
        onMouseOut = _this$props4.onMouseOut,
        onFocus = _this$props4.onFocus,
        onBlur = _this$props4.onBlur,
        disabled = _this$props4.disabled;
    var triggerComponent = React.Children.only(children);
    var triggerProps = triggerComponent.props;
    var props = {
      key: 'triggerComponent',
      onClick: createChainedFunction(triggerProps.onClick, onClick),
      'aria-describedby': get(speaker, ['props', 'id'])
    };

    if (!disabled) {
      if (isOneOf('click', trigger)) {
        props.onClick = createChainedFunction(this.handleToggle, props.onClick);
      }

      if (isOneOf('active', trigger)) {
        props.onClick = createChainedFunction(this.show, props.onClick);
      }

      if (isOneOf('hover', trigger)) {
        props.onMouseOver = createChainedFunction(this.onMouseOverListener, triggerProps.onMouseOver, onMouseOver);
        props.onMouseOut = createChainedFunction(this.onMouseOutListener, triggerProps.onMouseOut, onMouseOut);
      }

      if (isOneOf('focus', trigger)) {
        props.onFocus = createChainedFunction(this.handleDelayedShow, triggerProps.onFocus, onFocus);
        props.onBlur = createChainedFunction(this.handleDelayedHide, triggerProps.onBlur, onBlur);
      }
    }

    return [React.cloneElement(triggerComponent, props), React.createElement(Portal, {
      key: "portal"
    }, this.getOverlay())];
  };

  return OverlayTrigger;
}(React.Component);

OverlayTrigger.defaultProps = {
  trigger: ['hover', 'focus'],
  delayHide: 200,
  placement: 'bottomStart',
  rootClose: true
};
export default OverlayTrigger;
